@@ -34,7 +34,8 @@ from . import utilities
 # The prefix can be any word characters plus the dash character.
 # The suffix must be an underscore followed by the letter v and then the
 # semantic versioning version X.Y.Z and finally it must end with .yaml
-YAML_PATTERN = '^[\w-]*_v[\d]*\.[\d]*\.[\d]*\.yaml$'
+# 2/1/22 now using the official semver matching expression
+YAML_PATTERN = '^[\w-]*_v(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?.yaml$'
 
 # Logger for logs
 LOGGER = logging.getLogger('dax')
@@ -42,8 +43,14 @@ LOGGER = logging.getLogger('dax')
 # The default singularity run command that includes the cleanenv option and
 # binds an INPUTS and OUTPUTS. The command is included in the job_template
 # which will already have the variables set for $INDIR and $OUTDIR
-SINGULARITY_RUN = 'singularity run -e -H $INDIR -B $INDIR:/INPUTS -B $OUTDIR:/OUTPUTS'
-
+SINGULARITY_RUN = (
+    'singularity run --contain --cleanenv '
+    '--home $JOBDIR '
+    '--bind $INDIR:/INPUTS '
+    '--bind $OUTDIR:/OUTPUTS '
+    '--bind $JOBDIR:/tmp '
+    '--bind $JOBDIR:/dev/shm '
+    )
 
 class ParserArtefact:
     def __init__(self, path, resources, entity):
@@ -799,6 +806,7 @@ class Processor_v3(object):
 
             # TODO: optimize this to get resource list only once
             for vnum, vinput in enumerate(assr_inputs[v['input']]):
+                fname = None
                 robj = get_resource(assr._intf, vinput, resource)
 
                 # Get list of all files in the resource, relative paths
@@ -818,6 +826,9 @@ class Processor_v3(object):
                 if 'filepath' in cur_res:
                     fpath = cur_res['filepath']
                     res_path = resource + '/files/' + fpath
+
+                    # Get base file name to be downloaded
+                    fname = os.path.basename(fpath)
                 elif fmatch:
                     # Filter list based on regex matching
                     regex = utilities.extract_exp(fmatch, full_regex=False)
@@ -827,15 +838,36 @@ class Processor_v3(object):
                         LOGGER.debug('no matching files found on resource')
                         raise NeedInputsException('No Files')
 
-                    # Make a comma separated list of files
-                    uri_list = ['{}/files/{}'.format(resource, f) for f in file_list]
-                    res_path = ','.join(uri_list)
+                    if len(file_list) > 1:
+                        # Multiple files found, we only support explicit
+                        # declaration of fmulti==any1, which tells dax to use
+                        # any of the multiple files. We may later support
+                        # other options
+
+                        if 'fmulti' in cur_res and cur_res['fmulti'] == 'any1':
+                            LOGGER.debug('multiple files, fmulti==any1, using first found')
+                        else:
+                            LOGGER.debug('multiple files, fmulti not set')
+                            raise NeedInputsException(artk + ': multiple files')
+
+                    # Create the full path to the file on the resource
+                    res_path = '{}/files/{}'.format(resource, file_list[0])
+
+                    # Get just the filename for later
+                    fname = os.path.basename(file_list[0])
                 else:
+                    # We want the whole resource
                     res_path = resource + '/files'
 
+                    # Get just the resource name for later
+                    fname = resource
+
                 variable_set[k] = get_uri(assr._intf.host, vinput, res_path)
 
-                if len(assr_inputs[v['input']]) > 1:
+                if 'fdest' not in cur_res:
+                    # Use the original file/resource name
+                    fdest = fname
+                elif len(assr_inputs[v['input']]) > 1:
                     fdest = str(vnum) + cur_res['fdest']
                 else:
                     fdest = cur_res['fdest']
@@ -1203,7 +1235,8 @@ def parse_artefacts(csess, pets=[]):
 
     return artefacts
 
-
+# Returns the full URI for the resource_path as a child of input path which
+# can be either a scan or an assessor
 def get_uri(host, input_path, resource_path):
     if '/scans/' in input_path:
         uri_path = '{0}/data{1}/resources/{2}'.format(host, input_path, resource_path)
@@ -1214,7 +1247,8 @@ def get_uri(host, input_path, resource_path):
 
     return uri_path
 
-
+# Returns an xnat object for the resource that is a child of input_path 
+# which can be either a scan or an assessor
 def get_resource(xnat, input_path, resource):
     if '/scans/' in input_path:
         resource_path = '{0}/resources/{1}'
@@ -1226,14 +1260,16 @@ def get_resource(xnat, input_path, resource):
 
     return robj
 
-
+# Returns the processing type (proctype) as parsed from the aleady validated
+# yaml file name
 def parse_proctype(yaml_file):
     # At this point we assume the yaml file name is valid
     tmp = os.path.basename(yaml_file)
     tmp = tmp.rsplit('.')[-4]
     return tmp
 
-
+# Returns the processing version (procversion) as parsed from the aleady validated
+# yaml file name
 def parse_procversion(yaml_file):
     # At this point we assume the yaml file name is valid
     tmp = os.path.basename(yaml_file)
